using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MSW.Collections.Test
{
	public partial class FrmMain : Form
	{
		//	RingBuffer
		
		private MSW.Collections.Generic.RingBuffer<object> FBuffer = null;
	
		//	Update Log
		
		private Program.UpdateLogDelegate FUpdateLog = null;
		
		private void UpdateLog(string message)
		{
			string crlf = "\r\n";
		
			if (this.RTLog.Text.Length == 0)
			{
				crlf = "";
			}

			this.RTLog.AppendText(string.Format("{0}{1:HH:mm:ss:fff} : {2}", crlf, System.DateTime.Now, message));
			
			this.RTLog.ScrollToCaret();
		}

		//	Peek Buffer / Index

		#if DEBUG

		private MSW.Collections.RingBufferBase.PeekBufferDelegate FPeekBufferDelegate = null;

		#endif

		private void PeekBuffer(string message)
		{
			string crlf = "\r\n";

			if (this.RTPeekBuffer.Text.Length == 0)
			{
				crlf = "";
			}

			this.RTPeekBuffer.AppendText(string.Format("{0}{1}", crlf, message));
		}

		//	Status Update
		
		private void StatusUpdate()
		{
			//	Set Status
		
			this.CXOverflow.Checked = this.FBuffer.Overflow;
			
			this.CXEmpty.Checked = this.FBuffer.Empty;
			
			this.CXFull.Checked = this.FBuffer.Full;
		
			this.TBCount.Text = this.FBuffer.Count.ToString();
		
			//	Peek Buffer
		
			this.RTPeekBuffer.Clear();

			if (this.RBPeekBuffer.Checked)
			{
				#if DEBUG

				this.FBuffer.PeekBuffer(this.FPeekBufferDelegate);

				#else
			
				this.PeekBuffer("Not available in Release");
			
				#endif
			}
			else
			{
				int i = 0;
				
				foreach (object o in this.FBuffer)
				{
					this.PeekBuffer(string.Format("{0:00} : {1}", i++, o));	
				}
			}
		}

		//	Random

		private System.Random FRandom = null;

		//	Create Object

		private object CreateObject()
		{
			if (this.RBRandom.Checked)
			{
				return this.FRandom.Next(0, 1000);
			}
			else if (this.RBInteger.Checked)
			{
				int i = 0;

				int.TryParse(this.TBInteger.Text, out i);

				int j = i;

				if (this.CXIncrement.Checked)
				{
					i++;
				}

				this.TBInteger.Text = i.ToString();

				return j;
			}
			else
			{
				string text = this.TBItemText.Text;

				if (this.RBPlusRandom.Checked)
				{
					text += string.Format(" {0}", this.FRandom.Next(0, 1000));
				}
				else if (this.RBPlusInteger.Checked)
				{
					int i = 0;

					int.TryParse(this.TBInteger.Text, out i);

					text += string.Format(" {0}", i);

					if (this.CXIncrement.Checked)
					{
						i++;
					}

					this.TBInteger.Text = i.ToString();
				}

				return text;
			}
		}

		//	Get Application Icon

		[DllImport("kernel32.dll")]

		static extern IntPtr LoadLibrary(string lpFileName);

		[DllImport("kernel32.dll", SetLastError = true)]

		static extern bool FreeLibrary(IntPtr hModule);

		[DllImport("user32.dll")]

		private static extern IntPtr LoadIcon(IntPtr hInstance, IntPtr lpIconID);

		[DllImport("user32.dll")]

		extern static bool DestroyIcon(IntPtr handle);

		private Icon GetApplicationIcon()
		{
			try
			{
				System.Reflection.Assembly a = System.Reflection.Assembly.GetExecutingAssembly();

				IntPtr hLib = LoadLibrary(a.Location);

				if (hLib != IntPtr.Zero)
				{
					try
					{
						IntPtr hIcon = LoadIcon(hLib, (IntPtr)101);

						if (hIcon != IntPtr.Zero)
						{
							try
							{
								return Icon.FromHandle(hIcon);
							}
							finally
							{
								DestroyIcon(hIcon);
							}
						}
					}
					finally
					{
						FreeLibrary(hLib);
					}
				}
			}
			catch
			{
				return null;
			}

			return null;
		}

		//	Constructor

		public FrmMain()
		{
			this.Icon = this.GetApplicationIcon();
		
			//	Create Update Log Delegate
		
			this.FUpdateLog = new Program.UpdateLogDelegate(this.UpdateLog);

			#if DEBUG

			this.FPeekBufferDelegate = new MSW.Collections.RingBufferBase.PeekBufferDelegate(this.PeekBuffer);

			#endif

			//	Initialize
		
			InitializeComponent();
			
			//	Create Buffer
			
			this.FBuffer = new MSW.Collections.Generic.RingBuffer<object>(25);

			//	About

			this.FBuffer.About();

			//	Update Parameters
			
			this.TBSize.Text = this.FBuffer.Size.ToString();

			this.CXSilentOverflow.Checked = this.FBuffer.SilentOverflow;

			this.TBInsert.Text = (0).ToString();

			this.TBRemove.Text = (0).ToString();

			this.TBInteger.Text = (0).ToString();

			//	Create Random
			
			this.FRandom = new Random();
			
			//	Update Buffer Log

			this.StatusUpdate();
		}

		//	Form Event : Closing

		private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
		{
			this.FBuffer.Dispose();
			
			this.FBuffer = null;
		}

		//	Timer Event : Main

		private void TIMain_Tick(object sender, EventArgs e)
		{
			string sTimeout = "";
		
			#if MSW_DEMO

			sTimeout = string.Format("{0} : Timeout", this.FBuffer.DemoTimeout);
			
			#else
			
			sTimeout = string.Format("{0} : Timeout", "none");
		
			#endif

			if (this.LATimeout.Text.CompareTo(sTimeout) != 0)
			{
				this.LATimeout.Text = sTimeout;
			}

			Program.UpdateLog(this.FUpdateLog);
		}

		//	Link Clicked

		private void RTLog_LinkClicked(object sender, LinkClickedEventArgs e)
		{
			try
			{
				System.Diagnostics.Process.Start(e.LinkText);
			}
			catch
			{
				//	Link will not break
			}
		}

		//	Button Event : Set Size

		private const int MAX_SIZE = 50;

		private void BTSetSize_Click(object sender, EventArgs e)
		{
			try
			{
				int size = this.FBuffer.Size;
				
				int.TryParse(this.TBSize.Text, out size);

				if (size > MAX_SIZE)
				{
					throw new Exception(string.Format("In this application the maximum buffer size is {0} (can be adapted)", MAX_SIZE));
				}
			
				if (size != this.FBuffer.Size)
				{
					this.FBuffer.Size = size;

					Program.AddLog(string.Format("Set Size : {0}", this.FBuffer.Size));
				}
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Set Size : {0}", E.Message));
			}

			this.TBSize.Text = this.FBuffer.Size.ToString();

			this.StatusUpdate();
		}

		//	CheckBox Event : Silent Overflow

		private void CXSilentOverflow_CheckedChanged(object sender, EventArgs e)
		{
			try
			{
				this.FBuffer.SilentOverflow = this.CXSilentOverflow.Checked;

				Program.AddLog(string.Format("Silent Overflow : {0}", this.FBuffer.SilentOverflow));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Silent Overflow : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Insert At

		private void BTInsert_Click(object sender, EventArgs e)
		{
			int index = 0;

			try
			{
				int.TryParse(this.TBInsert.Text, out index);

				this.TBInsert.Text = index.ToString();

				object o = this.CreateObject();

				this.FBuffer.Insert(index, o);				

				Program.AddLog(string.Format("Insert at index {0} : {1}", index, o));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Insert at index {0} : {1}", index, E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Remove At

		private void BTRemove_Click(object sender, EventArgs e)
		{
			int index = 0;

			try
			{
				int.TryParse(this.TBRemove.Text, out index);

				this.TBRemove.Text = index.ToString();

				this.FBuffer.RemoveAt(index);

				Program.AddLog(string.Format("Remove at index {0}", index));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Remove at index {0} : {1}", index, E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Push Front

		private void BTPushFront_Click(object sender, EventArgs e)
		{
			try
			{
				object o = this.CreateObject();
			
				this.FBuffer.PushFront(o);

				Program.AddLog(string.Format("Push Front : {0}", o));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Push Front : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Pop Front

		private void BTPopFront_Click(object sender, EventArgs e)
		{
			try
			{
				Program.AddLog(string.Format("Pop Front : {0}", this.FBuffer.PopFront()));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Pop Front : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Push Back

		private void BTPushBack_Click(object sender, EventArgs e)
		{
			try
			{
				object o = this.CreateObject();

				this.FBuffer.PushBack(o);

				Program.AddLog(string.Format("Push Back : {0}", o));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Push Back : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Pop Back

		private void BTPopBack_Click(object sender, EventArgs e)
		{
			try
			{
				Program.AddLog(string.Format("Pop Back : {0}", this.FBuffer.PopBack()));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Pop Back : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Clear

		private void BTClear_Click(object sender, EventArgs e)
		{
			try
			{
				this.FBuffer.Clear();

				Program.AddLog("Cleared");
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Clear : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Add

		private void BTAdd_Click(object sender, EventArgs e)
		{
			try
			{
				object o = this.CreateObject();

				this.FBuffer.Add(o);

				Program.AddLog(string.Format("Add : {0}", o));
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Add : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Sort

		private void BTSort_Click(object sender, EventArgs e)
		{
			try
			{
				if (this.CXReverse.Checked)
				{
					this.FBuffer.ReverseSort();

					Program.AddLog("Reverse Sorted");
				}
				else
				{
					this.FBuffer.Sort();

					Program.AddLog("Sorted");
				}
			}
			catch (Exception E)
			{
				Program.AddLog(string.Format("Exception : Sort : {0}", E.Message));
			}

			this.StatusUpdate();
		}

		//	Button Click : Reset

		private void BTReset_Click(object sender, EventArgs e)
		{
			this.TBInteger.Text = (0).ToString();
			
			this.StatusUpdate();
		}
	
		//	RadioButton Checked Changed : Peek Buffer

		private void RBPeekBuffer_CheckedChanged(object sender, EventArgs e)
		{
			this.StatusUpdate();
		}

		//	RadioButton Checked Changed : Index

		private void RBIndex_CheckedChanged(object sender, EventArgs e)
		{
			this.StatusUpdate();
		}

		//	Button Click : Clear Log

		private void BTClearLog_Click(object sender, EventArgs e)
		{
			this.RTLog.Clear();
		}

		//	Button Click : About

		private void BTAbout_Click(object sender, EventArgs e)
		{
			this.RTLog.Clear();

			this.FBuffer.About();
		}

		//	Button Click : License

		private void BTLicense_Click(object sender, EventArgs e)
		{
			try
			{
				System.Diagnostics.Process.Start(@"..\..\..\License.html");
			}
			catch
			{
				Program.AddLog(".");

				Program.AddLog("Unable to show license, file not found.");

				Program.AddLog(".");

				Program.AddLog("Please, check the installation directory for the file :");

				Program.AddLog(".");

				Program.AddLog("   License.html");

				Program.AddLog(".");
			}
		}

		//	Button Click : Managing Software

		private void LAManagingSoftware_Click(object sender, EventArgs e)
		{
			try
			{
				System.Diagnostics.Process.Start(this.LAManagingSoftware.Text);
			}
			catch
			{
				//	Link will not break
			}
		}
	}
}