using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace MSW.Collections.Test
{
	static class Program
	{
		//	Trace Listener

		private class Listener : System.Diagnostics.TraceListener
		{
			public override void Write(string message)
			{
				Program.AddLog(message);
			}

			public override void WriteLine(string message)
			{
				Program.AddLog(message);
			}
		}

		//	Lock Buffers

		private static object LogLock = null;

		#if MSW_DEMO

		private static MSW.Collections.DemoLog LogBufferA = null;

		private static MSW.Collections.DemoLog LogBufferB = null;

		private static MSW.Collections.DemoLog LogBufferIn = null;

		private static MSW.Collections.DemoLog LogBufferOut = null;

		#else

			private static MSW.Collections.Generic.RingBuffer<string> LogBufferA = null;

			private static MSW.Collections.Generic.RingBuffer<string> LogBufferB = null;

			private static MSW.Collections.Generic.RingBuffer<string> LogBufferIn = null;

			private static MSW.Collections.Generic.RingBuffer<string> LogBufferOut = null;
				
		#endif

		//	Add Log
		
		public static void AddLog(string message)
		{
			lock (LogLock)
			{
				try
				{
					LogBufferIn.PushFront(message);
				}
				catch
				{
					//	Exception will not break
				}
			}
		}
		
		//	Update Log
		
		public delegate void UpdateLogDelegate(string message);

		public static void UpdateLog(UpdateLogDelegate updatelog)
		{
			//	Swap Buffers

			lock (LogLock)
			{
				if (LogBufferIn == LogBufferB)
				{
					LogBufferIn = LogBufferA;

					LogBufferOut = LogBufferB;
				}
				else
				{
					LogBufferIn = LogBufferB;

					LogBufferOut = LogBufferA;
				}
			}
			
			//	Update Log
			
			try
			{
				try
				{
					if (updatelog != null)
					{
						//	Check Overflow

						if (LogBufferOut.Overflow)
						{
							updatelog("[Log Overflow : Some messages are missing]");
						}

						//	Update Messages

						while (!LogBufferOut.Empty)
						{
							updatelog(LogBufferOut.PopBack());
						}
					}
				}
				finally
				{
					LogBufferOut.Clear();
				}
			}
			catch
			{
				//	Exception will not break
			}
		}
		
		//	Main

		[STAThread]

		static void Main()
		{
			LogLock = new object();
		
			try
			{
				#if MSW_DEMO

				LogBufferA = new MSW.Collections.DemoLog(1024);

				#else
				
					LogBufferA = new MSW.Collections.Generic.RingBuffer<string>(1024);

				#endif

				try
				{
					LogBufferA.SilentOverflow = true;

					#if MSW_DEMO

					LogBufferB = new MSW.Collections.DemoLog(1024);
					
					#else

						LogBufferB = new MSW.Collections.Generic.RingBuffer<string>(1024);

					#endif

					try
					{
						LogBufferB.SilentOverflow = true;
					
						LogBufferIn = LogBufferA;
						
						LogBufferOut = LogBufferB;	
					
						System.Diagnostics.Trace.Listeners.Clear();

						System.Diagnostics.Trace.Listeners.Add(new Listener());

						try
						{
							Application.EnableVisualStyles();

							Application.SetCompatibleTextRenderingDefault(false);

							Application.Run(new FrmMain());
						}
						finally
						{
							System.Diagnostics.Trace.Listeners.Clear();
						}
					}
					finally
					{
						LogBufferB.Dispose();

						LogBufferB = null;
					}
				}
				finally
				{
					LogBufferA.Dispose();

					LogBufferA = null;
				}
			}
			finally
			{
				LogLock = null;
			}
		}
	}
}